@@ -37,7 +37,7 @@
  *  OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-/* $Id: mhash.c,v 1.15 1999/10/10 20:23:32 nikos Exp $ */
+/* $Id: mhash.c,v 1.21 2000/02/12 13:49:01 nikos Exp $ */
 
 #include <stdlib.h>
 
@@ -53,9 +53,9 @@
 #include "mhash_md5.h"
 #include "mhash_sha1.h"
 #include "mhash_tiger.h"
-#include "rmd128.h"
-#include "rmd160.h"
+#include "mhash_ripemd.h"
 #include "gosthash.h"
+#include "mhash_md2.h"
 
 #define MAX_THREADS 256		/* After that pointers are being reused */
 
@@ -104,11 +104,13 @@ struct mhash_hash_entry {
 
 static mhash_hash_entry algorithms[] = {
 	MHASH_ENTRY(MHASH_CRC32, 4, 0),
+#ifndef _DISABLE_MD2
+	MHASH_ENTRY(MHASH_MD2, 16, 0),
+#endif
 	MHASH_ENTRY(MHASH_MD5, 16, 64),
 	MHASH_ENTRY(MHASH_SHA1, 20, 64),
 	MHASH_ENTRY(MHASH_HAVAL, HAVAL_FPTLEN >> 3, 128),
-	MHASH_ENTRY(MHASH_RIPEMD160, 160 >> 3, 64),
-	MHASH_ENTRY(MHASH_RIPEMD128, 128 >> 3, 64),
+	MHASH_ENTRY(MHASH_RIPEMD160, 20, 64),
 	MHASH_ENTRY(MHASH_TIGER, 192 >> 3, 64),
 	MHASH_ENTRY(MHASH_GOST, 32, 0),
 	MHASH_ENTRY(MHASH_CRC32B, 4, 0),
@@ -182,21 +184,23 @@ MHASH mhash_init_int(const hashid type, const MHASH thread)
 		state[thread] = malloc(sizeof(MD5_CTX));
 		MD5Init((void *) state[thread]);
 		break;
+#ifndef _DISABLE_MD2
+	case MHASH_MD2:
+		state[thread] = malloc(sizeof(MD2_CTX));
+		MD2Init((void *) state[thread]);
+		break;
+#endif
 	case MHASH_SHA1:
-		state[thread] = malloc(sizeof(SHA1_CTX));
-		SHA1Init((void *) state[thread]);
+		state[thread] = malloc(sizeof(SHA_CTX));
+		sha_init((void *) state[thread]);
 		break;
 	case MHASH_HAVAL:
 		state[thread] = malloc(sizeof(haval_state));
 		haval_start((void *) state[thread]);
 		break;
-	case MHASH_RIPEMD128:
-		state[thread] = malloc(4 * sizeof(word32));
-		MDinit_128((void *) state[thread]);
-		break;
 	case MHASH_RIPEMD160:
-		state[thread] = malloc(5 * sizeof(word32));
-		MDinit_160((void *) state[thread]);
+		state[thread] = malloc(sizeof(RIPEMD_CTX));
+		ripemd_init((void *) state[thread]);
 		break;
 	case MHASH_TIGER:
 		state[thread] = malloc(3 * sizeof(word64));
@@ -246,17 +250,19 @@ int mhash(MHASH thread, const void *plaintext, size_t size)
 	case MHASH_MD5:
 		MD5Update((void *) state[thread], plaintext, size);
 		break;
+#ifndef _DISABLE_MD2
+	case MHASH_MD2:
+		MD2Update((void *) state[thread], plaintext, size);
+		break;
+#endif
 	case MHASH_SHA1:
-		SHA1Update((void *) state[thread], plaintext, size);
+		sha_update( (void *) state[thread], (void*) plaintext, size);
 		break;
 	case MHASH_HAVAL:
 		haval_hash((void *) state[thread], plaintext, size);
 		break;
-	case MHASH_RIPEMD128:
-		MDfinish_128((void *) state[thread], plaintext, size, 0);
-		break;
 	case MHASH_RIPEMD160:
-		MDfinish_160((void *) state[thread], plaintext, size, 0);
+		ripemd_update( (void *) state[thread], (void*) plaintext, size);
 		break;
 	case MHASH_TIGER:
 		tiger(plaintext, size, (void *) state[thread]);
@@ -284,8 +290,20 @@ void *mhash_end(MHASH thread)
 	case MHASH_MD5:
 		rtmp = MD5Final((void *) state[thread]);
 		break;
+#ifndef _DISABLE_MD2
+	case MHASH_MD2:
+		digest =
+		    malloc(mhash_get_block_size(algorithm_given[thread]));
+
+		MD2Final(digest, (void *) state[thread]);
+		rtmp=digest;
+		break;
+#endif
 	case MHASH_SHA1:
-		rtmp = SHA1Final((void *) state[thread]);
+		sha_final((void *) state[thread]);
+		digest = malloc(SHA_DIGESTSIZE);
+		sha_digest( (void *) state[thread], digest);
+		rtmp=digest;
 		break;
 	case MHASH_HAVAL:
 		digest =
@@ -294,16 +312,10 @@ void *mhash_end(MHASH thread)
 		rtmp = digest;
 		break;
 	case MHASH_RIPEMD160:
-		digest = malloc(160 >> 3);
-		memcpy(digest, (void *) state[thread], 160 >> 3);
-		mhash_32bit_conversion(digest, 160 >> 5);
-		rtmp = digest;
-		break;
-	case MHASH_RIPEMD128:
-		digest = malloc(128 >> 3);
-		memcpy(digest, (void *) state[thread], 128 >> 3);
-		mhash_32bit_conversion(digest, 128 >> 5);
-		rtmp = digest;
+		ripemd_final((void *) state[thread]);
+		digest = malloc(RIPEMD_DIGESTSIZE);
+		ripemd_digest( (void *) state[thread], digest);
+		rtmp=digest;
 		break;
 	case MHASH_TIGER:
 		digest = malloc(192 >> 3);
@@ -403,8 +415,19 @@ void *hmac_mhash_end(MHASH thread)
 	case MHASH_MD5:
 		return_val = MD5Final((void *) state[thread]);
 		break;
+#ifndef _DISABLE_MD2
+	case MHASH_MD2:
+		digest =
+		    malloc(mhash_get_block_size(algorithm_given[thread]));
+		MD2Final(digest, (void *) state[thread]);
+		return_val = digest;
+		break;
+#endif
 	case MHASH_SHA1:
-		return_val = SHA1Final((void *) state[thread]);
+		digest = malloc(SHA_DIGESTSIZE);
+		sha_final((void *) state[thread]);
+		sha_digest( (void *) state[thread], digest);
+		return_val = digest;
 		break;
 	case MHASH_HAVAL:
 		digest =
@@ -413,13 +436,9 @@ void *hmac_mhash_end(MHASH thread)
 		return_val = digest;
 		break;
 	case MHASH_RIPEMD160:
-		digest = malloc(160 >> 3);
-		memcpy(digest, (void *) state[thread], 160 >> 3);
-		return_val = digest;
-		break;
-	case MHASH_RIPEMD128:
-		digest = malloc(128 >> 3);
-		memcpy(digest, (void *) state[thread], 128 >> 3);
+		digest = malloc(RIPEMD_DIGESTSIZE);
+		ripemd_final((void *) state[thread]);
+		ripemd_digest( (void *) state[thread], digest);
 		return_val = digest;
 		break;
 	case MHASH_TIGER:
