@@ -1,4 +1,5 @@
 #include <stdio.h>
+#include <string.h>
 
 #ifndef GAA_NO_WIN32
 #if defined(_WIN32) || defined(__WIN32__) || defined(WIN32) || defined(WINDOWS)
@@ -110,12 +111,13 @@ void gaa_help()
 	__gaa_helpsingle('c', "check", "", "check files.");
 	__gaa_helpsingle('b', "binary", "", "treat input files as binary.");
 	__gaa_helpsingle('t', "text", "", "treat input files as text.");
-	__gaa_helpsingle('o', "keymode", """kmode"" ", "keyword mode (8bit, hex, or hash). Hash is the default.");
+	__gaa_helpsingle('o', "keymode", """kmode"" ", "keyword mode""Use the --listkeygen parameter to view all supported modes.");
 	__gaa_helpsingle('i', "config", """config"" ", "Use configuration file FILE.");
 	__gaa_helpsingle('a', "algorithm", """algorithm"" ", "specify the algorithm. ""Use the --list parameter to show the supported algorithms.");
 	__gaa_helpsingle('n', "nolock", "", "Do not lock files.");
 	__gaa_helpsingle('l', "list", "", "Prints a list of the algorithms supported.");
 	__gaa_helpsingle('k', "key", """key"" ", "specify the key");
+	__gaa_helpsingle(0, "listkeygen", "", "Prints a list of the key generation algorithms supported.");
 	__gaa_helpsingle('h', "help", "", "prints this help");
 	__gaa_helpsingle('V', "version", "", "prints the version number");
 	__gaa_helpsingle('L', "license", "", "displays license information.");
@@ -133,19 +135,19 @@ typedef struct _gaainfo gaainfo;
 
 struct _gaainfo
 {
-#line 49 "shash.gaa"
+#line 52 "shash.gaa"
 	char **input;
-#line 48 "shash.gaa"
+#line 51 "shash.gaa"
 	int size;
-#line 41 "shash.gaa"
+#line 42 "shash.gaa"
 	char *keys;
-#line 36 "shash.gaa"
+#line 37 "shash.gaa"
 	int nolock;
-#line 32 "shash.gaa"
+#line 33 "shash.gaa"
 	char *algorithm;
-#line 29 "shash.gaa"
+#line 30 "shash.gaa"
 	char *config_file;
-#line 28 "shash.gaa"
+#line 29 "shash.gaa"
 	int config;
 #line 25 "shash.gaa"
 	char *kmode;
@@ -217,24 +219,25 @@ int gaa_error = 0;
 #define GAA_MULTIPLE_OPTION     3
 
 #define GAA_REST                0
-#define GAA_NB_OPTION           17
+#define GAA_NB_OPTION           18
 #define GAAOPTID_license	1
 #define GAAOPTID_version	2
 #define GAAOPTID_help	3
-#define GAAOPTID_key	4
-#define GAAOPTID_list	5
-#define GAAOPTID_nolock	6
-#define GAAOPTID_algorithm	7
-#define GAAOPTID_config	8
-#define GAAOPTID_keymode	9
-#define GAAOPTID_text	10
-#define GAAOPTID_binary	11
-#define GAAOPTID_check	12
-#define GAAOPTID_doublecheck	13
-#define GAAOPTID_hmac	14
-#define GAAOPTID_nosalt	15
-#define GAAOPTID_quiet	16
-#define GAAOPTID_verbose	17
+#define GAAOPTID_listkeygen	4
+#define GAAOPTID_key	5
+#define GAAOPTID_list	6
+#define GAAOPTID_nolock	7
+#define GAAOPTID_algorithm	8
+#define GAAOPTID_config	9
+#define GAAOPTID_keymode	10
+#define GAAOPTID_text	11
+#define GAAOPTID_binary	12
+#define GAAOPTID_check	13
+#define GAAOPTID_doublecheck	14
+#define GAAOPTID_hmac	15
+#define GAAOPTID_nosalt	16
+#define GAAOPTID_quiet	17
+#define GAAOPTID_verbose	18
 
 #line 168 "gaa.skel"
 
@@ -486,6 +489,7 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECK1STR("L", GAAOPTID_license);
 			GAA_CHECK1STR("V", GAAOPTID_version);
 			GAA_CHECK1STR("h", GAAOPTID_help);
+			GAA_CHECK1STR("", GAAOPTID_listkeygen);
 			GAA_CHECK1STR("l", GAAOPTID_list);
 			GAA_CHECK1STR("n", GAAOPTID_nolock);
 			GAA_CHECK1STR("t", GAAOPTID_text);
@@ -503,6 +507,7 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECKSTR("license", GAAOPTID_license);
 			GAA_CHECKSTR("version", GAAOPTID_version);
 			GAA_CHECKSTR("help", GAAOPTID_help);
+			GAA_CHECKSTR("listkeygen", GAAOPTID_listkeygen);
 			GAA_CHECKSTR("key", GAAOPTID_key);
 			GAA_CHECKSTR("list", GAAOPTID_list);
 			GAA_CHECKSTR("nolock", GAAOPTID_nolock);
@@ -555,23 +560,30 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
     {
 	case GAAOPTID_license:
 	OK = 0;
-#line 46 "shash.gaa"
+#line 49 "shash.gaa"
 { hash_license(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_version:
 	OK = 0;
-#line 45 "shash.gaa"
+#line 48 "shash.gaa"
 { hash_version(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_help:
 	OK = 0;
-#line 44 "shash.gaa"
+#line 47 "shash.gaa"
 { gaa_help(); exit(0); ;};
 
+		return GAA_OK;
+		break;
+	case GAAOPTID_listkeygen:
+	OK = 0;
+#line 46 "shash.gaa"
+{ list_keygen_algorithms(); exit(0); ;};
+
 		return GAA_OK;
 		break;
 	case GAAOPTID_key:
@@ -579,21 +591,21 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_key.arg1, gaa_getstr, GAATMP_key.size1);
 		gaa_index++;
-#line 42 "shash.gaa"
+#line 43 "shash.gaa"
 { gaaval->keys = GAATMP_key.arg1; ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_list:
 	OK = 0;
-#line 39 "shash.gaa"
+#line 40 "shash.gaa"
 { hash_list(); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_nolock:
 	OK = 0;
-#line 37 "shash.gaa"
+#line 38 "shash.gaa"
 { gaaval->nolock = 1 ;};
 
 		return GAA_OK;
@@ -603,7 +615,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_algorithm.arg1, gaa_getstr, GAATMP_algorithm.size1);
 		gaa_index++;
-#line 33 "shash.gaa"
+#line 34 "shash.gaa"
 { gaaval->algorithm = GAATMP_algorithm.arg1 ;};
 
 		return GAA_OK;
@@ -613,7 +625,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_config.arg1, gaa_getstr, GAATMP_config.size1);
 		gaa_index++;
-#line 30 "shash.gaa"
+#line 31 "shash.gaa"
 { gaaval->config=1; if (GAATMP_config.size1==1) gaaval->config_file = GAATMP_config.arg1 ;};
 
 		return GAA_OK;
@@ -686,7 +698,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		break;
 	case GAA_REST:
 		GAA_OPTIONALLIST_FILL(GAAREST_tmp.arg1, gaa_getstr, char*, GAAREST_tmp.size1);
-#line 50 "shash.gaa"
+#line 53 "shash.gaa"
 { gaaval->input = GAAREST_tmp.arg1; gaaval->size = GAAREST_tmp.size1 ;};
 
 		return GAA_OK;
@@ -714,7 +726,7 @@ int gaa(int argc, char **argv, gaainfo *gaaval)
     if(inited == 0)
     {
 
-#line 52 "shash.gaa"
+#line 55 "shash.gaa"
 { gaaval->quiet=1; gaaval->config=0; gaaval->kmode = NULL; gaaval->hmac = 0;
        gaaval->algorithm=NULL; gaaval->keys=NULL; gaaval->config_file=NULL; gaaval->input=NULL;
        gaaval->double_check=0; gaaval->nolock=0; gaaval->nosalt=0; gaaval->check=0; gaaval->text=0; 
@@ -888,7 +900,6 @@ int gaa_internal_get_next_str(FILE *file, gaa_str_node *tmp_str, int argc)
     tmp_str->str[i - 1] = 0;
 
     fseek(file,- 1, SEEK_CUR);
-//    printf("%d\n", ftell(file));
     
     return -1;
 }
